"use strict";(self.webpackChunk_JUPYTERLAB_CORE_OUTPUT=self.webpackChunk_JUPYTERLAB_CORE_OUTPUT||[]).push([[2595,8863],{58863:(e,t,r)=>{r.r(t),r.d(t,{MIME_TYPE:()=>d,RenderedJSON:()=>l,default:()=>h,rendererFactory:()=>c});var o=r(45494),n=r(32247),s=r(23190),a=r(43116),i=r(17029);const d="application/json";class l extends s.Widget{constructor(e){super(),this._rootDOM=null,this.addClass("jp-RenderedJSON"),this.addClass("CodeMirror"),this._mimeType=e.mimeType,this.translator=e.translator||n.nullTranslator}[o.Printing.symbol](){return()=>o.Printing.printWidget(this)}async renderModel(e){const{Component:t}=await Promise.all([r.e(262),r.e(4095),r.e(558),r.e(2122),r.e(7181),r.e(4433)]).then(r.bind(r,94095)),o=e.data[this._mimeType]||{},n=e.metadata[this._mimeType]||{};return null===this._rootDOM&&(this._rootDOM=(0,i.s)(this.node)),new Promise(((e,r)=>{this._rootDOM.render(a.createElement(t,{data:o,metadata:n,translator:this.translator,forwardedRef:()=>e()}))}))}onBeforeDetach(e){this._rootDOM&&(this._rootDOM.unmount(),this._rootDOM=null)}}const c={safe:!0,mimeTypes:[d],createRenderer:e=>new l(e)},h=[{id:"@jupyterlab/json-extension:factory",description:"Adds renderer for JSON content.",rendererFactory:c,rank:0,dataType:"json",documentWidgetFactoryOptions:{name:"JSON",primaryFileType:"json",fileTypes:["json","notebook","geojson"],defaultFor:["json"]}}]},17029:(e,t,r)=>{var o=r(61933);t.s=o.createRoot,o.hydrateRoot}}]);